@@ -104,13 +104,15 @@
 #'   (Default), \code{\link[ade4]{lingoes}}, and \code{\link[ade4]{cailliez}}.
 #'   See Details below.
 #'   
-#' @param dfname if the input data set is a \code{\linkS4class{genind}} object, 
-#'   specify the name of the data frame in the \code{\link[adegenet]{other}}
-#'   slot defining the population hierarchy. Defaults to
-#'   \code{"population_hierarchy"}
-#'   
 #' @param sep A single character used to separate the hierarchical levels. This
 #' defaults to "_".
+#' 
+#' @param filter \code{logical} When set to \code{TRUE}, mlg.filter will be run 
+#'   to determine genotypes from the distance matrix. It defaults to 
+#'   \code{FALSE}. You can set the parameters with \code{algorithm} and
+#'   \code{threshold} arguments. Note that this will not be performed when 
+#'   \code{within = TRUE}. Note that the threshold should be the number of
+#'   allowable substitutions if you don't supply a distance matrix.
 #'   
 #' @param missing specify method of correcting for missing data utilizing
 #'   options given in the function \code{\link{missingno}}. Default is
@@ -122,14 +124,16 @@
 #' @param quiet \code{logical} If \code{FALSE} (Default), messages regarding any
 #'   corrections will be printed to the screen. If \code{TRUE}, no messages will
 #'   be printed.
+#' @inheritParams mlg.filter
 #'   
 #' @return a list of class \code{amova} from the ade4 package. See 
 #'   \code{\link[ade4]{amova}} for details.
 #' 
 #' @details The poppr implementation of AMOVA is a very detailed wrapper for the
 #'   ade4 implementation. The output is an \code{\link[ade4]{amova}} class list 
-#'   that contains the results in the first four elements. The inputs are contained in the 
-#'   last three elements. The inputs required for the ade4 implementation are: 
+#'   that contains the results in the first four elements. The inputs are
+#'   contained in the last three elements. The inputs required for the ade4
+#'   implementation are:
 #'   \enumerate{
 #'   \item a distance matrix on all unique genotypes (haplotypes)
 #'   \item a data frame defining the hierarchy of the distance matrix 
@@ -166,6 +170,15 @@
 #'   \code{\link{cailliez}}. The correction of these distances should not 
 #'   adversely affect the outcome of the analysis.}
 #'   
+#'   \subsection{On Filtering:}{ Filtering multilocus genotypes is performed by
+#'   \code{\link{mlg.filter}}. This can necessarily only be done AMOVA tests 
+#'   that do not account for within-individual variance. The distance matrix used
+#'   to calculate the amova is derived from using \code{\link{mlg.filter}} with
+#'   the option \code{stats = "distance"}, which reports the distance between 
+#'   multilocus genotype clusters. One useful way to utilize this feature is to
+#'   correct for genotypes that have equivalent distance due to missing data. 
+#'   (See example below.)}
+#'   
 #' @keywords amova
 #' @aliases amova
 #' 
@@ -176,10 +189,11 @@
 #' 
 #' @seealso \code{\link[ade4]{amova}} \code{\link{clonecorrect}}
 #'   \code{\link{diss.dist}} \code{\link{missingno}}
-#'   \code{\link[ade4]{is.euclid}} \code{\link{sethierarchy}}
+#'   \code{\link[ade4]{is.euclid}} \code{\link{strata}}
 #' @export
 #' @examples
 #' data(Aeut)
+#' strata(Aeut) <- other(Aeut)$population_hierarchy[-1]
 #' agc <- as.genclone(Aeut)
 #' agc
 #' amova.result <- poppr.amova(agc, ~Pop/Subpop)
@@ -193,34 +207,65 @@
 #' amova.cc.test <- randtest(amova.cc.result)
 #' plot(amova.cc.test)
 #' amova.cc.test
+#' 
+#' # Example with filtering
+#' data(monpop)
+#' splitStrata(monpop) <- ~Tree/Year/Symptom
+#' poppr.amova(monpop, ~Symptom/Year) # gets a warning of zero distances
+#' poppr.amova(monpop, ~Symptom/Year, filter = TRUE, threshold = 0.1) # no warning
 #' }
 #==============================================================================#
 #' @importFrom ade4 amova is.euclid cailliez quasieuclid lingoes
 poppr.amova <- function(x, hier = NULL, clonecorrect = FALSE, within = TRUE, 
                         dist = NULL, squared = TRUE, correction = "quasieuclid", 
-                        dfname = "population_hierarchy", sep = "_", 
-                        missing = "loci", cutoff = 0.05, quiet = FALSE){
+                        sep = "_", filter = FALSE, threshold = 0, 
+                        algorithm = "farthest_neighbor", missing = "loci", 
+                        cutoff = 0.05, quiet = FALSE){
   if (!is.genind(x)) stop(paste(substitute(x), "must be a genind object."))
   if (is.null(hier)) stop("A population hierarchy must be specified")
   parsed_hier <- gsub(":", sep, attr(terms(hier), "term.labels"))
   full_hier <- parsed_hier[length(parsed_hier)]
-  
-  if (is.genclone(x)){
-    setpop(x) <- hier
-    other(x)[[dfname]] <- gethierarchy(x, hier, combine = FALSE)
-  } else {
-    if (!dfname %in% names(other(x))){
-      stop(paste(dfname, "is not present in the 'other' slot"))
+
+  setPop(x) <- hier
+  if (filter && (!within | all(ploidy(x) == 1) | !check_Hs(x) | x@type == "PA")){
+    
+    if (!is.genclone(x)){
+      x <- as.genclone(x)
     }
-    if (!full_hier %in% names(other(x)[[dfname]])){
-      hiers <- all.vars(hier)
-      if (!all(hiers %in% names(other(x)[[dfname]]))){
-        hier_incompatible_warning(hiers, df)
-      }
-      suppressWarnings(x <- splitcombine(x, hier = hiers, dfname = dfname, method = 2))
+    if (!is(x@mlg, "MLG")){
+      x@mlg <- new("MLG", x@mlg)
+    }
+    if (!quiet){
+      message("Filtering ...")
+      message("Original multilocus genotypes ... ", nmll(x, "original"))
+    }
+    if (is.null(dist)){
+      nulldist <- TRUE
+      dist <- diss.dist(x, percent = FALSE)
     } else {
-      pop(x) <- other(x)[[dfname]][[full_hier]]
+      nulldist <- FALSE
     }
+    filt_stats <- mlg.filter(x, threshold = threshold, algorithm = algorithm,
+                             distance = dist, stats = "ALL")
+    if (nulldist){
+      filt_stats$DISTANCE <- sqrt(filt_stats$DISTANCE)
+      squared <- FALSE
+    }
+    dist <- as.dist(filt_stats$DISTANCE)
+    # Forcing this. Probably should make an explicit method for this.
+    x@mlg@mlg["contracted"]    <- filt_stats$MLGS
+    x@mlg@distname             <- "diss.dist"
+    x@mlg@distalgo             <- algorithm
+    x@mlg@cutoff["contracted"] <- threshold
+    mll(x) <- "contracted"
+    if (!quiet) message("Contracted multilocus genotypes ... ", nmll(x))
+  }
+  if (clonecorrect){
+    x <- clonecorrect(x, strata = hier, keep = 1:length(all.vars(hier)))
+  }
+  if (within & all(ploidy(x) == 2) & check_Hs(x) & x@type != "PA"){
+    hier <- update(hier, ~./Individual)
+    x    <- pool_haplotypes(x)
   }
   # Treat missing data. This is a part I do not particularly like. The distance
   # matrix must be euclidean, but the dissimilarity distance will not allow
@@ -230,18 +275,11 @@ poppr.amova <- function(x, hier = NULL, clonecorrect = FALSE, within = TRUE,
   # missing to mean of the columns: indiscreet distances.
   # remove loci at cutoff
   # remove individuals at cutoff
-  if (clonecorrect){
-    x <- clonecorrect(x, hier = hier, keep = 1:length(all.vars(hier)))
-  }
-  if (within & ploidy(x) == 2 & check_Hs(x)){
-    hier <- update(hier, ~./Individual)
-    x    <- pool_haplotypes(x, dfname = dfname)
-  }
   x       <- missingno(x, type = missing, cutoff = cutoff, quiet = quiet)
-  hierdf  <- make_hierarchy(hier, other(x)[[dfname]])
+  hierdf  <- strata(x, formula = hier)
   xstruct <- make_ade_df(hier, hierdf)
   if (is.null(dist)){
-    xdist <- sqrt(diss.dist(clonecorrect(x, hier = NA), percent = FALSE))
+    xdist <- sqrt(diss.dist(clonecorrect(x, strata = NA), percent = FALSE))
   } else {
     datalength <- choose(nInd(x), 2)
     mlgs       <- mlg(x, quiet = TRUE)
@@ -252,11 +290,11 @@ poppr.amova <- function(x, hier = NULL, clonecorrect = FALSE, within = TRUE,
     } else if(length(dist) == mlglength){
       xdist <- dist
     } else {
-      msg <- paste("\nDistance matrix does not match the data.",
-      "\nUncorrected observations expected..........", nInd(x),
-      "\nClone corrected observations expected......", mlgs,
-      "\nObservations in provided distance matrix...", ceiling(sqrt(length(dist)*2)),
-      ifelse(within == TRUE, "\nTry setting within = FALSE.", "\n"))
+      msg <- paste("\nDistance matrix does not match the data.\n",
+      "\n\tUncorrected observations expected..........", nInd(x),
+      "\n\tClone corrected observations expected......", mlgs,
+      "\n\tObservations in provided distance matrix...", ceiling(sqrt(length(dist)*2)),
+      ifelse(within == TRUE, "\n\n\tTry setting within = FALSE.", "\n"))
       stop(msg)
     }
     if (squared){
@@ -271,15 +309,16 @@ poppr.amova <- function(x, hier = NULL, clonecorrect = FALSE, within = TRUE,
     } else {
       correct_fun <- match.fun(correct)
       if (correct == CORRECTIONS[2]){
-        cat("Distance matrix is non-euclidean.\n")
-        cat("Utilizing quasieuclid correction method. See ?quasieuclid for details.\n")
+        message("Distance matrix is non-euclidean.")
+        message("Utilizing quasieuclid correction method. See ?quasieuclid for details.")
         xdist <- correct_fun(xdist)        
       } else {
         xdist <- correct_fun(xdist, print = TRUE, cor.zero = FALSE)        
       }
     }
   }
-  xtab    <- t(mlg.table(x, bar = FALSE, quiet = TRUE, mlgsub = unique(mlg.vector(x))))
+  allmlgs <- unique(mlg.vector(x))
+  xtab    <- t(mlg.table(x, plot = FALSE, quiet = TRUE, mlgsub = allmlgs))
   xtab    <- as.data.frame(xtab)
   return(ade4::amova(samples = xtab, distances = xdist, structures = xstruct))
 }
